@@ -2,25 +2,14 @@
 ## (C) 2007, 2008, 2013 Muthiah Annamalai,
 ## Licensed under GPL Version 3
 ##
-## 1/25/08: move the scanner outside to another module.
-##          REPL-interpreter
-## 1/24/08: add Lexer line,col,file information.
-## 1/22/08: fix division operator, and function eval. add while.
-## 1/21/08: add newer operators.
-## 1/20/08: new lexer, and fix some evaluation errors.
-## 1/16/08: add bulitin functions. make space for conditionals.
-## 1/15/08: add simple function support.
-## 1/08/08: add visitor pattern to AST Objects.
 ## 
-## FEATURES: add array / list syntax.
 ## TODO: extract scanner and AST members into a module for sharing.
-## FIXME: grammar for expression maybe broke. precedence.
 import argparse
 from math import *
 import copy
 import os, sys, string, inspect
 import string
-
+import tamil
 from cmd import Cmd
 
 import time
@@ -39,9 +28,6 @@ from errors import RuntimeException, ParseException
 from runtime import  Environment, BuiltinFunction, \
  BlindBuiltins, DebugUtils
 
-# turtle graphics
-from EZTurtle import EZTurtle
-
 ## builtins
 import random
 
@@ -50,21 +36,24 @@ from ast import Expr, ExprCall, ExprList, Stmt, ReturnStmt, \
  BreakStmt, ContinueStmt, ElseStmt, IfStmt, WhileStmt, \
  ForStmt, AssignStmt, PrintStmt, EvalStmt, ArgList, \
  ValueList, Function, StmtList, Identifier, Number, \
- String
+ String, Boolean
 
 ## Parser
 from ExprsParser import Parser
 
 ## Transform / Visitor
-from transform import TreeWalker
+from transform import Visitor
 import collections
 
+def ezhil_version():
+        return 0.71
+
 def ezhil_copyright():
 	return "(C) 2007-2013 Muthiah Annamalai"
 
 # you can also, get your name here, its easy!
 def ezhil_credits():
-	return "Ezhil language was created by Muthiah Annamalai in 2007-2008"
+	return "Ezhil language, version %g, was created by Muthiah Annamalai in 2007-2008, and actively maintained in 2013."%(ezhil_version())
 
 def ezhil_license():
 	return "Licensed under GPL Version 3"
@@ -75,6 +64,25 @@ def ezhil_getitem(x,idx):
         return x.__getitem__(idx)
     return x[idx]
 
+def ezhil_islist( x ):
+    return Boolean(isinstance(x,list))
+
+def ezhil_isnumber( x ):
+    return Boolean(isinstance(x,Number) or type(x) in [int, float] )
+
+def ezhil_setitem(x,key,val):
+    #print("dispatching ezhil setitem",type(x),type(idx),x,idx,x[idx])
+    if ( isinstance(x,dict) or isinstance(x,list) or hasattr( x, '__setitem__') ):
+        x.__setitem__(key,val)
+    else:    
+        x[key]=val
+    return
+
+def ezhil_load(filename):
+    # load the file into the interpreter
+    # we have a symbol table and use it
+    pass
+
 # program name
 def get_prog_name(lang):
     prog_name=None
@@ -100,6 +108,17 @@ def get_prog_name(lang):
     
     return [prog_name, debug, dostdin]
 
+class NoClobberDict(dict):
+	""" dictionary structure with a set like mathematical structure.
+	    that way when you insert functions for ezhil library, we don't trample on each other
+	    by accidentally overwriting stuff"""
+	def __init__(self):
+		dict.__init__(self)
+	def __setitem__(self,key,val):
+		if ( self.has_key(key) ):
+			raise KeyError("Dictionary is getting clobbered; key "+key+" already present")
+		dict.__setitem__(self,key,val)
+
 ## Gandalf the Grey. One ring to rule them all.
 class Interpreter(DebugUtils):
     """ when you add new language feature, add a AST class 
@@ -107,12 +126,13 @@ class Interpreter(DebugUtils):
     def __init__(self,lexer, dbg = False):
         DebugUtils.__init__(self,dbg)
         self.debug = dbg
+        self.MAX_REC_DEPTH = 10000
         self.lexer=lexer
         self.ast=None
-        self.function_map = dict()#parsed functions
-        self.builtin_map = dict() #pointers to builtin functions
+        self.function_map = NoClobberDict()#parsed functions
+        self.builtin_map = NoClobberDict() #pointers to builtin functions
         self.call_stack = list() #call stack
-        
+        sys.setrecursionlimit( self.MAX_REC_DEPTH ) # have a large enough Python stack        
         ## use the default Exprs parser.
         lang_parser = Parser(self.lexer,self.function_map, \
                              self.builtin_map, self.debug )
@@ -122,6 +142,12 @@ class Interpreter(DebugUtils):
         self.install_builtins()
         self.install_blind_builtins()
 
+    def reset(self):
+        """ reset lexer and parser """
+        self.parser.reset()
+        self.lexer.reset()
+        return
+
     def change_parser(self,parser_ctor):
         """ change the parser to your-cool-language using
          a lambda function parser_ctor the constructor """
@@ -173,42 +199,47 @@ class Interpreter(DebugUtils):
     @staticmethod
     def file_close(*args):
         assert( len(args) == 1 )
-        getattr(args[0],'close')()
+        assert( hasattr(args[0],'close') )
+        args[0].close()
         return
     
     @staticmethod
     def file_read(*args):
         assert( len(args) == 1 )
-        return String( getattr(args[0],'read')())
+        assert( hasattr(args[0],'read') )
+        return String( args[0].read() )
     
     @staticmethod
     def file_readlines(*args):
         assert( len(args) == 1 )
-        return getattr(args[0],'readlines')()
+        assert( hasattr(args[0],'readlines') )
+        return args[0].readlines()
 
     @staticmethod
     def file_write(*args):
         assert( len(args) == 2 )
-        return getattr(args[0],'write')(args[1])
-
+        assert( hasattr(args[0],'write') )
+        return args[0].write(args[1])
+    
     @staticmethod
     def file_writelines(*args):
         assert( len(args) == 2 )
-        return getattr(args[0],'writelines')(args[1])
+        assert( hasattr(args[0],'writelines') )
+        return args[0].writelines(args[1])
     
     # marshalling    	    
     @staticmethod
     def RAWINPUT(args):
         op = raw_input(args)
         return String(op)
-
+    
     @staticmethod
     def INPUT(args):
         op = (raw_input(args))
         if ( isinstance(op,int) or isinstance(op,float) ):
             return Number(0.0+op)
         return String( op )
-
+    
     @staticmethod   
     def SPRINTF_worker(*args):        
         if ( len(args) < 1 ):
@@ -217,7 +248,7 @@ class Interpreter(DebugUtils):
         arg = tuple( args[1:] );
         opstr = fmt%arg;
         return opstr
-
+    
     @staticmethod
     def SPRINTF(*args):
         opstr = Interpreter.SPRINTF_worker(*args);
@@ -228,7 +259,7 @@ class Interpreter(DebugUtils):
         str_op = Interpreter.SPRINTF_worker(*args);
         print(str_op)
         return Number(len(str_op))
-
+    
     @staticmethod
     def ezhil_reverse(*args):
        if ( len(args) != 1 ): raise Exception('One argument alone expected to reverse function')
@@ -246,7 +277,7 @@ class Interpreter(DebugUtils):
              ezhil_sleep( 5 )
         else:
              ezhil_sleep( args[1] )
-	return
+        return
     
     def add_builtin(self,call_name,call_handle,nargin=1,ta_alias=None):	    
 	    # make sure you don't clobber yourself
@@ -284,8 +315,12 @@ class Interpreter(DebugUtils):
         self.builtin_map['coerce']=BlindBuiltins(coerce,'coerce',self.debug)
         self.builtin_map['compile']=BlindBuiltins(compile,'compile',self.debug)
         self.builtin_map['complex']=BlindBuiltins(complex,'complex',self.debug)
+
+        # special ezhil commands
         self.builtin_map['copyright']=BlindBuiltins(ezhil_copyright,'copyright',self.debug)
+        self.builtin_map['version']=BlindBuiltins(ezhil_version,'version',self.debug) 
         self.builtin_map['credits']=BlindBuiltins(ezhil_credits,'credits',self.debug)
+        
         self.builtin_map['delattr']=BlindBuiltins(delattr,'delattr',self.debug)
         self.builtin_map['dict']=BlindBuiltins(dict,'dict',self.debug)
         self.builtin_map['dir']=BlindBuiltins(dir,'dir',self.debug)
@@ -293,7 +328,7 @@ class Interpreter(DebugUtils):
         self.builtin_map['enumerate']=BlindBuiltins(enumerate,'enumerate',self.debug)
         self.builtin_map['eval']=BlindBuiltins(eval,'eval',self.debug)
         self.builtin_map['execfile']=BlindBuiltins(execfile,'execfile',self.debug)
-        self.builtin_map['exit']=BlindBuiltins(exit,'exit',self.debug)
+        #self.builtin_map['exit']=BlindBuiltins(exit,'exit',self.debug)
         self.builtin_map['file']=BlindBuiltins(file,'file',self.debug)
         self.builtin_map['filter']=BlindBuiltins(filter,'filter',self.debug)
         self.builtin_map['float']=BlindBuiltins(float,'float',self.debug)
@@ -305,8 +340,7 @@ class Interpreter(DebugUtils):
         self.builtin_map['hash']=BlindBuiltins(hash,'hash',self.debug)
         self.builtin_map['help']=BlindBuiltins(help,'help',self.debug)
         self.builtin_map['hex']=BlindBuiltins(hex,'hex',self.debug)
-        self.builtin_map['id']=BlindBuiltins(id,'id',self.debug)
-        #self.builtin_map['input']=BlindBuiltins(input,'input',self.debug)
+        self.builtin_map['id']=BlindBuiltins(id,'id',self.debug)        
         self.builtin_map['int']=BlindBuiltins(int,'int',self.debug)
         self.builtin_map['intern']=BlindBuiltins(intern,'intern',self.debug)
         self.builtin_map['isinstance']=BlindBuiltins(isinstance,'isinstance',self.debug)
@@ -314,23 +348,24 @@ class Interpreter(DebugUtils):
         self.builtin_map['iter']=BlindBuiltins(iter,'iter',self.debug)
         self.builtin_map['len']=BlindBuiltins(len,'len',self.debug)
         self.builtin_map['license']=BlindBuiltins(ezhil_license,'license',self.debug)
-        #self.builtin_map['list']=BlindBuiltins(list,'list',self.debug)
-        #self.builtin_map['locals']=BlindBuiltins(locals,'locals',self.debug)
         self.builtin_map['long']=BlindBuiltins(int,'long',self.debug)
         self.builtin_map['map']=BlindBuiltins(map,'map',self.debug)
-        self.builtin_map['max']=BlindBuiltins(max,'max',self.debug)
-        self.builtin_map['memoryview']=BlindBuiltins(memoryview,'memoryview',self.debug)
-        self.builtin_map['min']=BlindBuiltins(min,'min',self.debug)
+        #self.builtin_map['max']=BlindBuiltins(max,'max',self.debug)
+	try:
+	        self.builtin_map['memoryview']=BlindBuiltins(memoryview,'memoryview',self.debug)
+	except NameError as ie:
+		if(self.debug): print("Name Error:",str(ie))
+	
+        #self.builtin_map['min']=BlindBuiltins(min,'min',self.debug)
         self.builtin_map['next']=BlindBuiltins(next,'next',self.debug)
         self.builtin_map['object']=BlindBuiltins(object,'object',self.debug)
         self.builtin_map['oct']=BlindBuiltins(oct,'oct',self.debug)
-        self.builtin_map['open']=BlindBuiltins(open,'open',self.debug)
+        #self.builtin_map['open']=BlindBuiltins(open,'open',self.debug)
         self.builtin_map['ord']=BlindBuiltins(ord,'ord',self.debug)
-        self.builtin_map['pow']=BlindBuiltins(pow,'pow',self.debug)
+        #self.builtin_map['pow']=BlindBuiltins(pow,'pow',self.debug)
         self.builtin_map['property']=BlindBuiltins(property,'property',self.debug)
         self.builtin_map['quit']=BlindBuiltins(quit,'quit',self.debug)
-        self.builtin_map['range']=BlindBuiltins(range,'range',self.debug)
-        #self.builtin_map['raw_input']=BlindBuiltins(raw_input,'raw_input',self.debug)
+        self.builtin_map['range']=BlindBuiltins(range,'range',self.debug)        
         self.builtin_map['reduce']=BlindBuiltins(reduce,'reduce',self.debug)
         self.builtin_map['reload']=BlindBuiltins(reload,'reload',self.debug)
         self.builtin_map['repr']=BlindBuiltins(repr,'repr',self.debug)
@@ -351,8 +386,11 @@ class Interpreter(DebugUtils):
         self.builtin_map['vars']=BlindBuiltins(vars,'vars',self.debug)
         self.builtin_map['xrange']=BlindBuiltins(xrange,'xrange',self.debug)
         self.builtin_map['zip']=BlindBuiltins(zip,'zip',self.debug)
-        self.builtin_map['__getitem__']=BuiltinFunction(ezhil_getitem,"__getitem__",2)
 
+	# common/generic functions
+        self.builtin_map['__getitem__']=BuiltinFunction(ezhil_getitem,"__getitem__",2)
+        self.builtin_map['__setitem__']=BuiltinFunction(ezhil_setitem,"__setitem__",3)
+        
         #file-IO functions
         self.builtin_map["file_open"]=BlindBuiltins(Interpreter.file_open,"file_open")
         self.builtin_map["file_close"]=BuiltinFunction(Interpreter.file_close,"file_close")
@@ -369,12 +407,22 @@ class Interpreter(DebugUtils):
         self.builtin_map["assert"]=BuiltinFunction(Interpreter.ezhil_assert,"assert")
 	
         # sleep/pause
-	self.builtin_map["sleep"]=BuiltinFunction(ezhil_sleep,"sleep")
-	self.builtin_map["pause"]=BlindBuiltins(Interpreter.ezhil_pause,"pause")
+        self.builtin_map["sleep"]=BuiltinFunction(ezhil_sleep,"sleep")
+        self.builtin_map["pause"]=BlindBuiltins(Interpreter.ezhil_pause,"pause")
 	
-	# date/time
-	self.add_builtin("date_time",ezhil_date_time,nargin=0,ta_alias="தேதி_நேரம்")
+        # date/time
+        self.add_builtin("date_time",ezhil_date_time,nargin=0,ta_alias="தேதி_நேரம்")
+	self.add_builtin("time",time.time,nargin=0,ta_alias="நேரம்")
+	self.add_builtin("ctime",time.ctime,nargin=1,ta_alias="cநேரம்")
+	self.add_builtin("clock",time.time,nargin=0)
+
+	# islist, isnumber predicates
+	self.add_builtin("islist",ezhil_islist,nargin=1,ta_alias="பட்டியலா")
+	self.add_builtin("isnumber",ezhil_isnumber,nargin=1,ta_alias="எண்ணா")	
 	
+        # get tamil letters
+        self.add_builtin("get_tamil_letters",tamil.get_letters,nargin=1,ta_alias="தமிழ்_எழுத்துக்கள்")	
+
         # random functions
         aslist = True;
         self.builtin_map["choice"]=BlindBuiltins(random.choice,"choice",self.debug,aslist)
@@ -413,26 +461,25 @@ class Interpreter(DebugUtils):
 
         self.builtin_map["max"]=BuiltinFunction(max,"max",2)
         self.builtin_map["min"]=BuiltinFunction(min,"min",2)
-        self.builtin_map["exit"]=BuiltinFunction(min,"exit",1)
+        #self.builtin_map["exit"]=BuiltinFunction(min,"exit",1)
 
-        # turtle functions
-        turtle_attrib = EZTurtle.functionAttributes();
-        for nargs,fcnName in list(turtle_attrib.items()):
-            for vv in fcnName:
-                turtlefcn = "turtle_"+vv;
-                if ( self.debug ): print(nargs, vv)
-                if ( nargs == -1 ):
-                    self.builtin_map[turtlefcn] = BlindBuiltins(getattr(EZTurtle, vv),vv,self.debug)
-                else:
-                    self.builtin_map[turtlefcn] = BuiltinFunction( getattr( EZTurtle, vv ), turtlefcn, nargs )
-        
-        #string functions
-#       for x in dir(string):
-#            if ( x.__str__()[0] != '_' ):
-#                name = x.__str__()
-#                print "self.builtin_map[\"%s\"] = BuiltinFunction(getattr(string,\"%s\"),\"%s\")"%(name,name,name)
-        
+        # turtle functions - optional - 
+	try:
+		# turtle graphics
+		from EZTurtle import EZTurtle
 
+		turtle_attrib = EZTurtle.functionAttributes();
+		for nargs,fcnName in list(turtle_attrib.items()):
+			for vv in fcnName:
+				turtlefcn = "turtle_"+vv;
+				if ( self.debug ): print(nargs, vv)
+				if ( nargs == -1 ):
+					self.builtin_map[turtlefcn] = BlindBuiltins(getattr(EZTurtle, vv),vv,self.debug)
+				else:
+					self.builtin_map[turtlefcn] = BuiltinFunction( getattr( EZTurtle, vv ), turtlefcn, nargs )
+	except ImportError as ie:
+		if ( self.debug ): print "Cannot Import EZTurtle module; ignoring for now"
+        
     	self.builtin_map["ascii_letters"] = BuiltinFunction(string.ascii_letters,"ascii_letters",0)
     	self.builtin_map["ascii_lowercase"] = BuiltinFunction(string.ascii_lowercase,"ascii_lowercase",0)
     	self.builtin_map["ascii_uppercase"] = BuiltinFunction(string.ascii_uppercase,"ascii_uppercase",0)
@@ -445,12 +492,12 @@ class Interpreter(DebugUtils):
     	self.builtin_map["capitalize"] = BuiltinFunction(string.capitalize,"capitalize",1)
     	self.builtin_map["capwords"] = BuiltinFunction(string.capwords,"capwords",1)
     	self.builtin_map["center"] = BuiltinFunction(string.center,"center",1)
-    	self.builtin_map["count"] = BuiltinFunction(string.count,"count",1)
+    	self.builtin_map["count_string"] = BuiltinFunction(string.count,"count",1)
     	self.builtin_map["digits"] = BuiltinFunction(string.digits,"digits",1)
     	self.builtin_map["expandtabs"] = BuiltinFunction(string.expandtabs,"expandtabs",1)
     	self.builtin_map["find"] = BuiltinFunction(string.find,"find",2)
     	self.builtin_map["hexdigits"] = BuiltinFunction(string.hexdigits,"hexdigits",1)
-    	self.builtin_map["index"] = BuiltinFunction(string.index,"index",2)
+    	self.builtin_map["index_string"] = BuiltinFunction(string.index,"index",2)
     	self.builtin_map["index_error"] = BuiltinFunction(string.index_error,"index_error",1)
     	self.builtin_map["join"] = BuiltinFunction(string.join,"join",1)
     	self.builtin_map["joinfields"] = BuiltinFunction(string.joinfields,"joinfields",1)
@@ -479,38 +526,38 @@ class Interpreter(DebugUtils):
     	self.builtin_map["whitespace"] = BuiltinFunction(string.whitespace,"whitespace",1)
     	self.builtin_map["zfill"] = BuiltinFunction(string.zfill,"zfill",2)
     	
-        #add list methods - first argument, when required, is always a list obj
+	# get/set methods are handled by generic __getitem__ and __setitem__
+	
+        # list methods - first argument, when required, is always a list obj
         self.builtin_map["append"] = BuiltinFunction(list.append,"append",2)
         self.builtin_map["insert"] = BuiltinFunction(list.insert,"insert",3)
         self.builtin_map["index"] = BuiltinFunction(list.index,"index",2)
-        self.builtin_map["list"] = BuiltinFunction(list,"list",0)
-        self.builtin_map["pop"] = BuiltinFunction(list.pop,"pop",1)
+        self.builtin_map["list"] = BuiltinFunction(list,"list",0)        
         self.builtin_map["sort"] = BuiltinFunction(list.sort,"sort",1)
         self.builtin_map["count"]= BuiltinFunction(list.count,"count",2)
         self.builtin_map["extend"]= BuiltinFunction(list.extend,"extend",2)
         self.builtin_map["reverse"]= BuiltinFunction(Interpreter.ezhil_reverse,"reverse",1)
-        self.builtin_map["get"]= BuiltinFunction(list.__getitem__,"get",2)
+	self.builtin_map["pop_list"] = BuiltinFunction(list.pop,"pop",1)
+	
+	# dictionary methods - first argument, when required, is always a dict obj
+        self.builtin_map["clear"]= BuiltinFunction(dict.clear,"clear",1)
+        self.builtin_map["copy"]= BuiltinFunction(dict.copy,"copy",1)
+        self.builtin_map["fromkeys"]= BuiltinFunction(dict.fromkeys,"fromkeys",1)
+        self.builtin_map["has_key"]= BuiltinFunction(dict.has_key,"has_key",1)
+        self.builtin_map["items"]= BuiltinFunction(dict.items,"items",1)
+        self.builtin_map["iteritems"]= BuiltinFunction(dict.iteritems,"iteritems",1)
+        self.builtin_map["iterkeys"]= BuiltinFunction(dict.iterkeys,"iterkeys",1)
+        self.builtin_map["itervalues"]= BuiltinFunction(dict.itervalues,"itervalues",1)
+        self.builtin_map["keys"]= BuiltinFunction(dict.keys,"keys",1)
+        self.builtin_map["pop_dict"]= BuiltinFunction(dict.pop,"pop",1)
+        self.builtin_map["popitem"]= BuiltinFunction(dict.popitem,"popitem",1)
+        self.builtin_map["setdefault"]= BuiltinFunction(dict.setdefault,"setdefault",1)
+        self.builtin_map["update"]= BuiltinFunction(dict.update,"update",1)
+        self.builtin_map["values"]= BuiltinFunction(dict.values,"values",1)
+        #self.builtin_map["viewitems"]= BuiltinFunction(dict.viewitems,"viewitems",1)
+        #self.builtin_map["viewkeys"]= BuiltinFunction(dict.viewkeys,"viewkeys",1)
+        #self.builtin_map["viewvalues"]= BuiltinFunction(dict.viewvalues,"viewvalues",1)
         
-        # #dictionary methods - 
-        # self.builtin_map["clear"]= BuiltinFunction(dict.clear,"clear",1)
-        # self.builtin_map["copy"]= BuiltinFunction(dict.copy,"copy",1)
-        # self.builtin_map["fromkeys"]= BuiltinFunction(dict.fromkeys,"fromkeys",1)
-        # self.builtin_map["get"]= BuiltinFunction(dict.get,"get",1)
-        # self.builtin_map["has_key"]= BuiltinFunction(dict.has_key,"has_key",1)
-        # self.builtin_map["items"]= BuiltinFunction(dict.items,"items",1)
-        # self.builtin_map["iteritems"]= BuiltinFunction(dict.iteritems,"iteritems",1)
-        # self.builtin_map["iterkeys"]= BuiltinFunction(dict.iterkeys,"iterkeys",1)
-        # self.builtin_map["itervalues"]= BuiltinFunction(dict.itervalues,"itervalues",1)
-        # self.builtin_map["keys"]= BuiltinFunction(dict.keys,"keys",1)
-        # self.builtin_map["pop"]= BuiltinFunction(dict.pop,"pop",1)
-        # self.builtin_map["popitem"]= BuiltinFunction(dict.popitem,"popitem",1)
-        # self.builtin_map["setdefault"]= BuiltinFunction(dict.setdefault,"setdefault",1)
-        # self.builtin_map["update"]= BuiltinFunction(dict.update,"update",1)
-        # self.builtin_map["values"]= BuiltinFunction(dict.values,"values",1)
-        # self.builtin_map["viewitems"]= BuiltinFunction(dict.viewitems,"viewitems",1)
-        # self.builtin_map["viewkeys"]= BuiltinFunction(dict.viewkeys,"viewkeys",1)
-        # self.builtin_map["viewvalues"]= BuiltinFunction(dict.viewvalues,"viewvalues",1)
-
         return True
 
     def __repr__(self):
@@ -528,13 +575,14 @@ class Interpreter(DebugUtils):
 
     def evaluate(self):
         env = Environment( self.call_stack, self.function_map, \
-                               self.builtin_map, self.debug )
+                               self.builtin_map, self.debug, int(self.MAX_REC_DEPTH/10) )
         env.call_function("__toplevel__") ##some context
         return self.ast.evaluate(env)
 
     def evaluate_interactive(self,env = None):
         ## use a persistent environment for interactive interpreter
         if ( not env ):
+            if ( self.debug ): print "creating a new environment"
             env = Environment( self.call_stack, self.function_map, \
                                    self.builtin_map, self.debug )
             env.call_function("__toplevel__") ##some context
@@ -547,11 +595,15 @@ class Interpreter(DebugUtils):
 # refactor out REPL for ezhil and exprs
 class REPL(Cmd):
 	def __init__(self,lang, lexer, parse_eval, debug=False):
+		""" @lexer is language lexical analyzer
+		    @parse_eval is the language interpreter with builtins, runtime, parser etc..
+		    @lang is a descriptive string,
+		    @debug the boolean """
 		Cmd.__init__(self)
 		## ala-Python like
 		self.banner = """எழில் - ஒரு தமிழ் நிரலாக்க மொழி (Tue Jul  2 20:22:25 EDT 2013)
-Ezhil : A Tamil Programming Language, (C) 2007-2013
-Type "help", "copyright", "credits" or "license" for more information."""
+Ezhil : A Tamil Programming Language - version %g, (C) 2007-2013
+Type "help", "copyright", "credits" or "license" for more information."""%ezhil_version()
 		
 		self.lang = lang
 		self.lexer = lexer
@@ -560,7 +612,7 @@ Type "help", "copyright", "credits" or "license" for more information."""
 		self.line_no = 1
 		self.env = None ## get the first instance from evaluate_interactive
 		self.cmdloop()
-	
+    
 	def parseline(self,line):
 		arg,cmd = "",""
 		if line in ["exit","help","EOF","copyright","credits","license"]:
@@ -579,6 +631,7 @@ Type "help", "copyright", "credits" or "license" for more information."""
 		pass
 	
 	def default(self,line):
+		""" REPL is carried out primarily through this callback from the looping construct """
 		self.line_no += 1
 		
 		if ( self.debug ): print("evaluating line", line)
@@ -599,6 +652,7 @@ Type "help", "copyright", "credits" or "license" for more information."""
 		except Exception as excep:
 			print("Exception in code, at line %d,  \"%s\" \n >>>>>>> %s "%(self.line_no-1,line,str(excep)))
                 ## clear tokens in lexer
+		if ( self.debug ): print self.env
 		self.lexer.tokens = list()
 		self.update_prompt()
 	
@@ -617,4 +671,3 @@ Type "help", "copyright", "credits" or "license" for more information."""
 			sys.exit( 0 )
 		
 		return
-	
