@@ -3,11 +3,17 @@
 ## (C) 2008, 2013 Muthiah Annamalai,
 ## Licensed under GPL Version 3
 ## Ezhil PyPi Package
-import os, sys, string, tempfile
+
+from Interpreter import ezhil_version as version
+from Interpreter import ezhil_credits as credits
+from Interpreter import ezhil_copyright as copyright
+
 from Interpreter import Interpreter, REPL, Lex, get_prog_name
+
 from ezhil_parser import EzhilParser
 from ezhil_scanner import EzhilLex
 from errors import RuntimeException, ParseException
+
 from .ezhil import ezhil_file_REPL
 from .ezhil import EzhilInterpreter, EzhilFileExecuter, EzhilInterpExecuter
 from .ezhil import ezhil_interactive_interpreter as start
