@@ -1,5 +1,6 @@
 #!/usr/bin/python
-##
+# -*- coding: utf-8 -*-
+## 
 ## (C) 2007, 2008 Muthiah Annamalai,
 ## Licensed under GPL Version 3
 ## 
@@ -59,7 +60,8 @@ class Identifier:
  Line %d, col %d"%( self.id, self.line,self.col ) )
         return None
 
-    def visit_identifier(self, walker):
+    def visit(self, walker):
+        """ visitor - do something with a identifier """
         walker.visit_identifier(self)
         return
 
@@ -79,7 +81,7 @@ class String:
     def evaluate(self,env):
         return self.string
 
-    def visit_string(self, walker):
+    def visit(self, walker):
         walker.visit_string(self)
         return
 
@@ -105,7 +107,7 @@ class Number:
     def evaluate(self,env):
         return self.num
 
-    def visit_number(self, walker):
+    def visit(self, walker):
         walker.visit_number(self)
         return
 
@@ -117,6 +119,20 @@ class Boolean(Number):
             return "True"
         return "False"
 
+class Dict(dict):
+    def __init__(self):
+        dict.__init__(self)
+
+    def base_evaluate(self,env):
+        rval = {}
+        for x,y in self.items():
+            rval.update({ x.evaluate( env ): y.evaluate( env )} )
+        return rval
+    
+    def evaluate(self,env):
+        """ how do you evaluate dictionaries? just return the favor """
+        return self.base_evaluate( env )
+
 ## FIXME: implement arrays
 class Array(list):
     def __init__(self):
@@ -167,7 +183,7 @@ class ExprCall:
             raise RuntimeException("undefined function: %s near ( %d, %d )"%(self.fname, self.line, self.col) )
         return rval
 
-    def visit_expr_call(self,walker):
+    def visit(self,walker):
         walker.visit_expr_call(self)
         return
     
@@ -184,11 +200,11 @@ class ExprList:
     def evaluate(self,env):
         """evaluate  a, b, c ... z to a string"""
         z = []
-        for exp in self.exprs:
-            z.append(exp.evaluate(env))
+        for exp_itr in self.exprs:
+            z.append(exp_itr.evaluate(env))
         return ", ".join(map(str,z))
-
-    def visit_expr_list(self, walker):
+    
+    def visit(self, walker):
         walker.visit_expr_list(self)
         return    
 
@@ -235,14 +251,14 @@ class Stmt:
             if ( fval > 0.0 ):
                 rval = True
             ## all other cases later.
-        except Exception as e:
+        except Exception as pyEx:
             """ objects where is_true_value() is not supported """
-            print(e)
-            raise RuntimeException(e)
+            print(pyEx)
+            raise RuntimeException(pyEx)
         self.dbg_msg('Is True Value? ' + str(rval) + str(val.__class__) )
         return rval
-
-    def visit_stmt( self, walker):
+    
+    def visit( self, walker):
         walker.visit_stmt( self )
         return
     
@@ -266,8 +282,8 @@ class UnaryExpr(Stmt):
         term=self.term.evaluate(env)
         if ( self.debug ): print(term, type(term))
         if self.unaryop.kind in Token.UNARYOP:
-            tval = Expr.normalize_values( term, env)
-            if ( self.debug ): print(tval, type(tval), tval2, type(tval2))
+            tval = Expr.normalize_values( self, term, env)
+            if ( self.debug ): print(tval, type(tval))
             term = self.do_unaryop( tval )
         else:
             raise RuntimeException(" unknown Unary Operation - "+str(self.unaryop)+" not supported")
@@ -283,6 +299,10 @@ class Expr(Stmt):
         self.binop=op
         self.next_expr=next_expr        
 
+    def __len__(sef):
+        """ expr is unit length always, as opposed to ExprList which is a n-len thing """
+        return 1
+    
     def __repr__(self):
         return "\n\t [Expr[ "+ str(self.term)+"] " + \
                Token.token_types[self.binop.kind] + \
@@ -358,37 +378,43 @@ class Expr(Stmt):
         return val
 
     @staticmethod
-    def normalize_values(term, env ):
+    def normalize_values( obj, term, env ):
         if ( hasattr(term,'evaluate') ):
-            if ( term.__class__ == Number ):
+            if ( isinstance(term,Number) ): #work for both Number, and derived Boolean class
                 tval = term.num
-            elif ( term.__class__ == String ):
-                tval = term.string
+            elif ( isinstance(term,String) ):
+                tval = term.string                            
             else:
+                ## print term.__class__,term,str(term)
                 ## possibly leads to inf- recursion
                 ## tval = term.evaluate( env )
-                raise RuntimeException( " unknown clause to evaluate @ "+self.get_pos());
+                raise RuntimeException( " cannot normalize token; unknown clause,"+str(term)+", to evaluate @ "+obj.get_pos());
+#        elif isinstance(term,list) and len(term) == 1:
+#            tval = term[0]
         else:
-            tval = (term) #float cast not required.
+            tval = term #float cast not required.
         return tval
-
+    
     def evaluate(self,env):
         term=self.term.evaluate(env)
         if ( self.debug ): print term, type(term)
         if self.binop.kind in Token.BINOP:
             tnext = self.next_expr.evaluate(env)
-            tval = Expr.normalize_values( term, env)
-            tval2 = Expr.normalize_values( tnext, env)
+            tval = Expr.normalize_values( self, term, env)
+            tval2 = Expr.normalize_values( self, tnext, env)
             if ( self.debug ): print tval, type(tval), tval2, type(tval2)
-            term = self.do_binop(tval,
-                                 tval2,
-                                 self.binop.kind)
+            try:
+                term = self.do_binop(tval,
+                                     tval2,
+                                     self.binop.kind)
+            except Exception as binOp_Except:
+                raise RuntimeException("binary operation "+str(self.term)+str(self.binop)+str(self.next_expr)+" failed with exception "+str(binOp_Except))
         else:
             raise RuntimeException(" unknown Binary Operation - Binary operation "+str(self.binop)+" not supported")
         if ( self.debug ): print "term = ",term, term.__class__
         return term
 
-    def visit_expr(self, walker):
+    def visit(self, walker):
         walker.visit_expr(self)
         return
 
@@ -407,7 +433,7 @@ class ReturnStmt(Stmt):
         env.set_retval(rhs)
         return rhs
     
-    def visit_return_stmt(self, walker):
+    def visit(self, walker):
         walker.visit_return_stmt(self)
         return
 
@@ -424,7 +450,7 @@ class BreakStmt(Stmt):
         env.set_break()
         return None
     
-    def visit_break_stmt(self, walker):
+    def visit(self, walker):
         walker.visit_break_stmt(self)
         return
 
@@ -441,7 +467,7 @@ class ContinueStmt(Stmt):
         env.set_continue()
         return None
     
-    def visit_continue_stmt(self, walker):
+    def visit(self, walker):
         walker.visit_continue_stmt(self)
         return
 
@@ -457,7 +483,7 @@ class ElseStmt(Stmt):
     def evaluate(self,env):
         return self.stmt.evaluate(env)
     
-    def visit_else_stmt(self,walker):
+    def visit(self,walker):
         walker.visit_else_stmt(self)
         return
 
@@ -516,7 +542,7 @@ class IfStmt(Stmt):
         # its perfectly legal to not have an else statement
         return rval 
     
-    def visit_if_elseif_stmt(self,walker):
+    def visit(self,walker):
         walker.visit_if_elseif_stmt(self)
         return
 
@@ -546,7 +572,7 @@ class WhileStmt(Stmt):
         self.dbg_msg("exiting While-stmt with rval="+str(rval))
         return rval
 
-    def visit_while_stmt(self,walker):
+    def visit(self,walker):
         walker.visit_while_stmt(self)
         return
 
@@ -605,7 +631,7 @@ class ForStmt(Stmt):
         self.dbg_msg("exiting For-stmt with rval="+str(rval))
         return rval
 
-    def visit_for_stmt(self,walker):
+    def visit(self,walker):
         walker.visit_for_stmt(self)
         return
 
@@ -647,7 +673,7 @@ class AssignStmt(Stmt):
             return rhs
         raise Exception("Unknown assign operator @ "+self.get_pos())
     
-    def visit_assign_stmt(self, walker):
+    def visit(self, walker):
         walker.visit_assign_stmt(self)
         return
 
@@ -672,7 +698,7 @@ class PrintStmt(Stmt):
         self.do_printop(env)
         return None
     
-    def visit_print_stmt(self, walker):
+    def visit(self, walker):
         walker.visit_print_stmt(self)
         return
 
@@ -689,7 +715,7 @@ class EvalStmt(Stmt):
     def evaluate(self,env):
         return self.expr.evaluate(env)
 
-    def visit_eval_stmt(self, walker):
+    def visit(self, walker):
         walker.visit_eval_stmt(self)
         return
 
@@ -711,7 +737,7 @@ class ArgList:
     def __repr__(self):
         return "\n\t [ArgList["+ ",".join(map(str,self.args))+"]]"
 
-    def visit_arg_list(self,walker):
+    def visit(self,walker):
         walker.visit_arg_list(self)
         return
 
@@ -743,7 +769,7 @@ class ValueList:
     def __repr__(self):
         return "\n\t [ValueList["+ ",".join(map(str,self.args))+"]]"
 
-    def visit_value_list(self,walker):
+    def visit(self,walker):
         walker.visit_value_list(self)
         return
 
@@ -774,7 +800,8 @@ class StmtList(Stmt):
             rval = stmt.evaluate(env)
         return rval
 
-    def visit_stmt_list(self,walker):
+    def visit(self,walker):
+        """ visit stmt list method """
         walker.visit_stmt_list(self)
         return
 
@@ -827,6 +854,7 @@ class Function(Stmt):
         env.return_function(self.name)
         return  rval
 
-    def visit_function(self,walker):
+    def visit(self,walker):
         walker.visit_function(self)
         return
+
