@@ -31,7 +31,7 @@ from ast import Expr, UnaryExpr, ExprCall, ExprList, Stmt, ReturnStmt, \
  BreakStmt, ContinueStmt, ElseStmt, IfStmt, WhileStmt, DoWhileStmt, \
  ForStmt, AssignStmt, PrintStmt, EvalStmt, ArgList, \
  ValueList, Function, StmtList, Identifier, Number, \
- String, Array
+ String, Array, Dict
 
 ## use exprs language parser
 from ExprsParser import Parser
@@ -46,7 +46,6 @@ class EzhilParser(Parser):
                 raise RuntimeException("Cannot find Ezhil lexer class")
         Parser.__init__(self,lexer,fcn_map,builtin_map,dbg)
 
-
     def factory(lexer,fcn_map,builtin_map, dbg = False):
         """ Factory method """
         return EzhilParser(lexer,fcn_map,builtin_map, dbg)
@@ -154,7 +153,7 @@ class EzhilParser(Parser):
                 break
             else:
                 self.inside_if = False
-                raise ParseError("SWITCH-CASE-OTHERWISE statement syntax is messed up")
+                raise ParseException("SWITCH-CASE-OTHERWISE statement syntax is messed up")
             ptok = self.peek()
             self.dbg_msg("parsing SWITCH-CASE next bits "+str(ptok))
         self.match( EzhilToken.END )
@@ -210,7 +209,7 @@ class EzhilParser(Parser):
                 break
             else:
                 self.inside_if = False
-                raise ParseError("If-Else-If statement syntax is messed up")
+                raise ParseException("If-Else-If statement syntax is messed up")
             ptok = self.peek()
             self.dbg_msg("parsing -IF next bits "+str(ptok))
         self.match( EzhilToken.END )
@@ -228,6 +227,8 @@ class EzhilParser(Parser):
             self.dbg_msg('enter->return: <expression>')
             ret_tok = self.dequeue()
             [l,c]=ret_tok.get_line_col();
+            if ( not self.parsing_function ):
+                raise ParseException( "return statement outside of function body "+str(ret_tok))
             rstmt = ReturnStmt(self.expr(),l,c,self.debug)
             self.dbg_msg("return statement parsed")
             return rstmt
@@ -248,10 +249,11 @@ class EzhilParser(Parser):
                 self.currently_parsing.pop()
             else:
                 exp = pass_in_ATexpr
+                pass_in_ATexpr = None #use it just once
             if( self.debug ): print ("return from valuelist ",str(exp))
             ptok = self.peek();
             if ( ptok.kind == EzhilToken.IF ):                
-                return self.parseIfStmt(exp)            
+                return self.parseIfStmt(exp)
             elif ( ptok.kind ==  EzhilToken.WHILE ):
                 ## @ ( expr ) while { body } end               
                self.loop_stack.append(True)
@@ -277,7 +279,7 @@ class EzhilParser(Parser):
                 self.dbg_msg("foreach-statement")
                 # convert to a for statement - building Ezhil AST - transformations
                 if not isinstance( exp[1], Identifier ):
-                    raise ParseError(" FOR-EACH statement "+str(foreach_tok) )
+                    raise ParseException(" FOR-EACH statement "+str(foreach_tok) )
                 foreach_iter = exp[1];
                 iter = Identifier("__"+foreach_iter.id,l=0,c=-1);
                 eq_token = EzhilLexeme("=",EzhilToken.EQUALS)
@@ -375,7 +377,7 @@ class EzhilParser(Parser):
                 return AssignStmt( lhs, assign_tok, rhs, l, c, self.debug)
             self.currently_parsing.pop()
             return EvalStmt( lhs, l, c, self.debug )
-        raise ParseException("parsing Statement, unkown operators" + str(ptok))
+        raise ParseException("parsing Statement, unknown operators" + str(ptok))
     
     def function(self):
         """ def[kw] fname[id] (arglist) {body} end[kw] """
@@ -552,9 +554,23 @@ class EzhilParser(Parser):
                 vallist = self.valuelist()
                 val=ExprCall( val, vallist, l, c, self.debug )
             elif ( ptok.kind ==  EzhilToken.LSQRBRACE ):
-                ## indexing a array type variable or ID
+                ## indexing a array type variable or ID                
+                [l,c] = ptok.get_line_col()
+                ## replace with a call to __getitem__
+                exp = self.factor();
+                if ( hasattr(exp,'__getitem__') ):
+                    VL2 = ValueList([val,exp[0]],l,c,self.debug)
+                else:
+                    # when exp is a expression
+                    VL2 = ValueList([val,exp],l,c,self.debug)
+                val = ExprCall( Identifier("__getitem__",l,c), VL2,l,c,self.debug)
+                for itr in range(1,len(exp)):
+                    VL2 = ValueList([val,exp[itr]],l,c,self.debug)
+                    val = ExprCall( Identifier("__getitem__",l,c), VL2,l,c,self.debug)
+                #raise ParseException("array indexing implemented"+str(ptok));
+            elif ( ptok.kind ==  EzhilToken.LCURLBRACE ):
                 val=None
-                raise ParseException("arrays not implemented"+str(ptok));
+                raise ParseException("dictionary indexing implemented"+str(ptok));
         elif tok.kind ==  EzhilToken.STRING :
             str_tok = self.dequeue()
             [l,c] = str_tok.get_line_col()
@@ -563,11 +579,32 @@ class EzhilParser(Parser):
             unop = self.dequeue();
             [l, c] = unop.get_line_col()
             val=Expr(Number(0),unop,self.term(),l,c,self.debug);
+        elif tok.kind == EzhilToken.LCURLBRACE:
+            # creating a list/dictionary expression
+            dict_start = self.dequeue();
+            val = Dict()
+            while( True ):
+                if ( self.peek().kind == EzhilToken.RCURLBRACE ):
+                    break;
+                exprkey = self.expr()
+                tok_colon = self.match(EzhilToken.COLON)
+                exprval = self.expr()
+                val.update( {exprkey : exprval}  )
+                if self.debug : print(self.peek().__class__,self.peek())
+                if ( self.peek().kind == EzhilToken.RCURLBRACE ):
+                    break
+                else:
+                    assert( self.peek().kind == EzhilToken.COMMA)
+                    self.dequeue()
+            assert( self.peek().kind == EzhilToken.RCURLBRACE )
+            list_end = self.dequeue()
         elif tok.kind == EzhilToken.LSQRBRACE:
             # creating a list/array expression
             list_start = self.dequeue();
             val = Array()
             while( True ):
+                if ( self.peek().kind == EzhilToken.RSQRBRACE ):
+                    break;
                 exprval = self.expr()
                 val.append( exprval  )
                 if self.debug : print(self.peek().__class__,self.peek())
